import pytest

@pytest.mark.asyncio
async def test_context_session_open_and_close(Parser, useragent):
    async with Parser.create(useragent=useragent) as p:
        assert not p.session.closed
    assert p.session.closed

@pytest.mark.asyncio
async def test_classic_session_open_and_close(Parser, useragent):
    p = Parser(useragent=useragent)
    assert not p.session.closed

    await p.close()
    assert p.session.closed



# this hack allows pytest to find astana_hub package without installing package inside pipenv.
import sys
sys.path.append('../astana_hub')

import os
import pytest
import astana_hub


# не разобрался я как правильно ее отдавать так чтобы она и работала т отдалась объектои Parser именно и тирдаун у нее был правильный
# @pytest.fixture
# async def astana_hub_parser():
#     """Instance of astana_hub Parser object with some random useragent"""
#     async with astana_hub.Parser.create(useragent="very unsuspicious useragent!") as ah:
#         return ah

@pytest.fixture
def useragent():
    """sample useragent"""
    return "Mozilla/5.0 (X11; Linux i686 on x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/55.0.2909.25 Safari/537.36"

@pytest.fixture
def Parser():
    """astana_hub Parser class object"""
    return astana_hub.Parser


@pytest.fixture
def company_list_html():
    """Copy of https://astanahub.com/account/company/  html page as of 30/10/2022."""
    with open(os.path.join(os.path.dirname(__file__), "company_list.html"), "r") as f:
        return f.read()

@pytest.fixture
def company_list_page_52_html():
    """Copy of https://astanahub.com/account/company/?page=52  html page as of 30/10/2022."""
    with open(os.path.join(os.path.dirname(__file__), "company_list_52.html"), "r") as f:
        return f.read()



import asyncio
from dataclasses import dataclass, field
from contextlib import asynccontextmanager
import logging

import aiohttp
from bs4 import BeautifulSoup


@dataclass
class Page:
    items: list
    page_num: int
    total_pages: int


@dataclass
class Item:
    title: str
    href: str
    description: str
    img_src: str = field(default=None, repr=False)
    full_description: str = field(default=None, repr=False)


class Parser:
    BASE_URL = 'https://astanahub.com'
    DEFAULT_USERAGENT = "Mozilla/5.0 (X11; Linux i686 on x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/55.0.2909.25 Safari/537.36"

    TAGS = {
        "company": ('', 'tag_startup', 'tag_it_company', 'tag_corp_partner', 'tag_techpark', 'tag_ts_member'),
        "user": ('', 'tag_intern', 'tag_it_specialist', 'tag_investor', 'tag_international_agent'),
    }

    def __init__(self, useragent=None, proxy=None):
        self.useragent = useragent if useragent else Parser.DEFAULT_USERAGENT
        self.proxy = proxy if proxy else {}

        self.session = aiohttp.ClientSession()
        self.logger = logging.getLogger('astana_hub.parser')

        assert type(self.useragent) is str, f"invalid {useragent=}"
        assert type(self.proxy) is dict, f"unsupported {proxy=}"

    @asynccontextmanager
    async def create(*args, **kwargs):
        parser = Parser(*args, **kwargs)
        try:
            yield parser
        finally:
            await parser.close()

    async def close(self):
        if self.session is not None and not self.session.closed:
            await self.session.close()

    def _soup_factory(html):
        return BeautifulSoup(html, "lxml")

    def _parse_html(html):
        soup = Parser._soup_factory(html)
        company_list = soup.find("div", class_="company-list")

        assert company_list

        items = []
        for card in company_list.find_all("a", class_="card"):
            item = Item(
                title = card.find('h4').text,
                href = card.attrs['href'],
                description = card.find('span').text,
                img_src = card.find('img') and card.find('img').attrs['src'],                      # optional
                full_description = card.find('p') and card.find('p').text.strip()                 # optional
            )
            items.append(item)

        # if there is only one page, ther is no pagination block and 1, 1 is valid
        page_num, total_pages = 1, 1

        pagination = soup.find("ul", class_="pagination-list")
        if pagination:
            li_list = pagination.find_all("li")
            if li_list:
                el = li_list[-1].find("a", class_="paginator")
                if el:
                    total_pages = int(el.text)

            current_a = pagination.find("a", class_="active")
            if current_a:
                page_num = int(current_a.text)

        return Page(items=items, page_num=page_num, total_pages=total_pages)

    async def get_page(self, page_type, page=1, tag=''):
        assert page_type in ('company', 'user'), f"unsupported {page_type=} for get_page request!"
        assert tag in Parser.TAGS[page_type], f"unsupported {tag=} for get_page request with {page_type=}!"

        url = f"{Parser.BASE_URL}/account/{page_type}/"
        params = {"page": page, "tag": tag}
        headers = {'User-Agent': self.useragent, 'Referer': url}

        async with self.session.get(url, params=params, **self.proxy) as response:
            self.logger.info(f"get_page request ({page_type=}, {params=}, {headers=})")
            assert response.status == 200, f"{response.status=}"

            html = await response.text()
            page_obj = Parser._parse_html(html)

            assert page_obj.page_num == page
            return page_obj

    async def get_company_page(self, page=1, tag=''):
        return await self.get_page(page_type='company', page=page, tag=tag)

    async def get_user_page(self, page=1, tag=''):
        return await self.get_page(page_type='user', page=page, tag=tag)

    async def parse_all(self, page_type, tag='', sleep=1):
        items = []
        page_num = 1
        while True:
            page = await self.get_page(page_type=page_type, page=page_num,  tag=tag)
            items.extend( page.items )

            if page.page_num == page.total_pages:
                return items

            assert page_num < page.total_pages, f"we somehow got here: {page_num=} {page.total_pages=}"

            await asyncio.sleep(sleep)
            page_num += 1



import pytest

def test_company_list_parse_html_properties(Parser, company_list_html):
    page = Parser._parse_html(company_list_html)

    assert len(page.items) == 30
    assert page.total_pages == 52
    assert page.page_num == 1

def test_company_list_parse_html_page_52_properties(Parser, company_list_page_52_html):
    page = Parser._parse_html(company_list_page_52_html)

    assert len(page.items) == 1
    assert page.total_pages == 52
    assert page.page_num == 52

def test_company_list_parse_single_card(Parser, company_list_html):
    page = Parser._parse_html(company_list_html)

    item = page.items[0]

    assert item.href == '/account/company/2464/'
    assert item.title == 'Abay Academy'
    assert item.description == 'IT-компания'



from setuptools import setup, find_packages
import json
import pathlib


HERE = pathlib.Path(__file__).parent.resolve()

def locked_requirements(section):
    """Look through the 'Pipfile.lock' to fetch requirements by section."""
    with open('Pipfile.lock') as pip_file:
        pipfile_json = json.load(pip_file)

    if section not in pipfile_json:
        print(f"{section} section missing from Pipfile.lock")
        return []

    return [package + detail.get('version', "") for package, detail in pipfile_json[section].items()]


setup(
    name="astana_hub",
    # version="0.0.1",   default set via setup.cfg

    description="Get Networking cards from https://astanahub.com/ with ease!",
    long_description=(HERE / "README.md").read_text(encoding="utf-8"),
    long_description_content_type="text/markdown",

    url="https://github.com/vladzen13/astana_hub",

    author="Vladislav Zenin",
    author_email="vladzen13@yandex.ru",

    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",

        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
    ],

    # keywords="sample, setuptools, development",  # Optional

    # package_dir={"": "astana_hub"},

    packages=find_packages(),

    python_requires=">=3.9, <4",
    install_requires=locked_requirements('default'),
    extras_require={
        "dev": locked_requirements('develop'),
    },

    # tests_require=['pytest'],   ????????????? from pinnacle/ is it valid ?

    # package_data={  # Optional
    #     "sample": ["package_data.dat"],
    # },

    # data_files=[("my_data", ["data/data_file"])],  # Optional

    # entry_points={  # Optional
    #     "console_scripts": [
    #         "sample=sample:main",
    #     ],
    # },

    project_urls={
        "Bug Reports": "https://github.com/vladzen13/astana_hub/issues",
        "Source": "https://github.com/vladzen13/astana_hub/",
        # "Funding": "https://donate.pypi.org",
        # "Say Thanks!": "http://saythanks.io/to/example",
    },
)
